#' ---
#' title: "R Basics"
#' author: "Andreas Blombach, Philipp Heinrich and Stefan Evert"
#' date: "08.10.2019"
#' output:
#'  html_document:
#'    theme: readable
#'    highlight: tango
#'    toc: true
#'    toc_float: true
#'    fig_width: 10
#' ---
#' 

#' ## R as a calculator
#' Try entering some basic calculations!
#' 
#' ## Objects
#' To work with data, we  use *objects*. Objects are stored in
#' the current *environment*. Single values, data structures and
#' functions are all objects.
#' 
#' To access an object, it needs a name. This name should start
#' with a letter (after that, you're free to use whatever
#' combination of letters, numbers and some special characters
#' (like dots or underscores) you like).
my_first_object <- 42 # the arrow assigns the value 42 to my_first_object

#' Execute the line above by placing the cursor somewhere in it
#' and pressing CTRL+Enter (or by clicking "Run" in RStudio).
#' 
#' Notice that it will now appear in the environment.
#' 
#' You can now access the object's value by entering its name:
my_first_object

#' Create a few more objects. You can also assign the value of an
#' existing object to a new object (thereby copying it), do
#' calculations with objects and assign the result of these to
#' objects. (What happens when you multiply an object by itself
#' and assign the result to the same object?)
#' 

#' You can also see the objects in the current environment by
#' executing this function:
ls()

#' To get rid of an object, use rm():
rm(my_first_object)

#' To get rid of *all* objects in the current environment:
rm(list = ls())

#' ## Functions
#' Functions like these can be recognised by the round brackets.
#' 
#' A function has a name (the characters before the brackets) and
#' may have different arguments (between the brackets), separated
#' by commas. Arguments may be either optional or obligatory.
#' 
#' For example, the following function produces five random
#' numbers between 1 and 10:
sample(1:10, 5)

#' To take a sample with replacement (allowing the same number
#' to be drawn multiple times), we can specifiy the optional
#' argument *replace*:
sample(1:10, 11, replace = TRUE)

#' To look up what a function does and how it works, you can
#' access the built-in documentation by typing ? followed by the
#' function's name: ?sample
#' 
#' If you enter a function's arguments in the exact same order
#' as seen in its documentation, you don't need to specify the
#' names of its arguments. If you do specify them, however, you
#' are free to enter them in any order you want:
sample(replace = TRUE, size = 5, x = 1:100)

#' ## Installing and loading packages
#' R already provides quite a lot of functions, but sooner or
#' later, you'll need some more ...
#' 
#' A package is a collection of functions and/or data sets,
#' usually for a certain range of applications (e.g. plotting,
#' linear mixed-effects models, corpus analysis, ...).
#' 
#' When packages are installed, they are stored locally (e.g. on
#' a hard drive). The set of installed packages can be thought of
#' as a library: if you need a certain package in your current
#' session, you can check it out (thus activating it).
#' 
#' To install a package (or several, by providing a vector of
#' package names):
#' install.packages("name_of_package")
#' install.packages(c("package1", "package2", "package3"))
#' 
#' By default, dependencies are also installed (= packages which
#' are required for your new package to work properly).
#' 
#' To activate an installed package:
#' library("name_of_package")
#' library(name_of_package)
#' 
#' (For whatever reason, quotation marks are optional in this
#' case.)
#' 
#' You can also use RStudio to install, update, activate and
#' deactivate packages.
#' 
#' A much more extensive tutorial (useful even for advanced
#' users):
#' https://www.datacamp.com/community/tutorials/rpackages-guide
#' 
#' We'll need some packages later, so let's activate them:
library(tidyverse)
library(readxl)
library(data.table)


#' ## Logical Operators
#' To check if values are equal, if one is greater than another
#' etc., we need logical operators.
a <- 5
b <- 2
#' Are a and b equal?
a == b

#' Is a greater than b?
a > b

#' Also:
a >= b # greater or equal
a < b # smaller
a <= b # smaller or equal
a != b # not equal

#' On its own, the exclamation mark ("not") negates an expression
!(a < b) # returns TRUE if the following expression is FALSE

#' We can use & (AND) and | (OR) to combine conditions:
a > b & a != b # TRUE if both sides are TRUE
a < b | a > b # TRUE if at least one of the sides is TRUE

#' If we want to know if only one of both sides is TRUE, we need
#' XOR (excluding OR):
xor(a < b, a > b) # TRUE
xor(a > b, a != b) # FALSE (since both conditions are TRUE)

#' Side note: There's also && und || which behave somewhat
#' differently -- you'll need these for if statements.
#' 


#' ## Data types
#' There are lots of types of data in R. Luckily, we won't need
#' all of them.
#' 
#' These are the most important basic types:
#' 
#' - numbers (integers and doubles)
#' - logical values (TRUE and FALSE)
#' - characters (= strings)
#' 
#' (Use typeof() to determine the basic type of objects.)
#' 
#' Data structures:
#' 
#' - vectors
#' - factors (ordered and unordered)
#' - matrices
#' - data sets (data.frame, tibble and data.table)
#' - lists
#' 
#' ### Vectors
#' Probably the most import data structure in R, a vector
#' contains elements of the same basic type (for different types,
#' you'll need lists).
#' 
test_vector <- c(3, 4.6, 64, 42)
test_vector

#' Vectors of characters/strings or logical values are also
#' possible:
strings <- c("colourless", "green", "ideas")
strings
logicals <- c(TRUE, FALSE, TRUE, TRUE, FALSE)
logicals

#' To get the number of elements in a vector, use `length()`:
length(test_vector)

#' We can use square brackets to access elements of a vector:
test_vector[1]
test_vector[2]
strings[2]

#' We can also use vectors of numbers to access several elements:
test_vector[1:3] # elements 1-3
logicals[2:4]
strings[c(1, 3)]

#' Vectors can be part of new vectors:
test_vector2 <- c(48, 120, 5, 32)
new_vector <- c(test_vector, test_vector2); new_vector

#' To sort a vector, use `sort()`:
sort(new_vector)
sort(new_vector, decreasing = TRUE)

#' Mathematical operators and many functions are vectorised which
#' means that when applied to a vector, you get a vector in
#' return:
new_vector + 2 # every element + 2
new_vector * 3 # every element multiplied by 3
sqrt(new_vector) # square root
round(new_vector)
round(new_vector, 1) # optional second argument: number of decimal places


#' R provides some useful methods to search inside of vectors:
#' 
#' Use `%in%` to check if a vector contains a certain value:
48 %in% new_vector

#' `which()` returns the position(!) of elements that meet your
#' conditions (by using logical operators, see above):
which(new_vector == 42)
which(new_vector > 40)

#' Since this is a vector itself, you can use it to access the
#' elements of the original vector by their position:
new_vector[which(new_vector > 40)]
#' But this might be a little easier:
new_vector[new_vector > 40]

#' You can also combine conditions:
new_vector[new_vector > 40 & new_vector < 50]

#' ### Data sets
#' We can import data in a number of ways. R prefers CSV files,
#' but there are packages to read in other file formats (Excel,
#' SPSS, JSON, etc.).
#' 
#' #### Reading in data
#' Those with some R experience probably already know
#' `read.table()`, `read.csv()`, `read.csv2()` etc.
#' Alternatively, you can read in a data set as a tibble which is
#' a little faster. For really large files, the `data.table`
#' package offers the function `fread()`.
#' 
#' Which function is best suited to read in a specific file
#' depends on the file format and the file's formatting (field
#' separator, decimal point, etc.). (Exception: `fread()`.
#' `fread()` doesn't care and usually figures this out by itself.)
#' 
#' For CSV files in European format (semicolon as field separator,
#' comma as decimal point), use `read_csv2()`:
gen_blogs <- read_csv2("data/Genitive_DWDS_Blogs.csv")
gen_blogs
#' To specify data types for certain columns:
gen_blogs <- read_csv2("data/Genitive_DWDS_Blogs.csv",
                       col_types = "?ii")
gen_blogs

#' The first argument is a file path. Since the folder "data" is
#' located in my current working directory, I don't need to
#' specify the full/absolute path.
#' 
#' Alternatively, you can use `file.choose()` to select a file:
#' read_csv2(file.choose())
#' 
#' Try it out!
#' 
#' There's also read_csv() for classic CSV files (comma as field
#' separator, `.` as decimal point), read_tsv() for files with
#' tab stops as field separators, and read_delim(), the parent
#' function where you can specify everything yourself.
#' 
#' RStudio offers some options to read in files a little more
#' comfortably: File -> Import Dataset
#' 
#' - From Text (base)...: base R functions: read.table() etc.: data.frame
#' - From Text (readr)...: tidyverse/readr style: tibble
#' - From Excel...: Excel files (tibble)
#' - From SPSS...: SPSS files (tibble)
#' - From SAS...: SAS files (tibble)
#' - From Stata...: Stata files (tibble)
#' 
#' Having selected fitting options to import your data and having
#' clicked "Import", you can see the R command on the console.
#' You can then copy it to your script to speed up the process in
#' the future.
#' 
#' Example: Opening an Excel file:
gen_blogs <- read_excel("data/Genitive_DWDS_Blogs.xlsx")
gen_blogs


#' If you've got your own data with you, now's the time to try to
#' open it!


#' #### Accessing parts of a data set
#' To access a column (usually a statistical variable), enter the
#' data set's name, followed by a Dollar sign and the name of the
#' column. We get a vector of values (let's not display all of
#' them by using `head()`):
gen_blogs$Lemma %>% head()
gen_blogs$s.Genitiv %>% head()

#' Just as with vectors, you can use square brackets to subset
#' a data set. You just have to provide two values: row and
#' column.
gen_blogs[2, 3] # second row, third column
gen_blogs[3, c(1, 3)] # third row, columns 1 and 3
gen_blogs[3,] # third row, all columns (don't forget the comma!)
gen_blogs[, 3] # third column

#' To select certain columns, `select()` is also useful:
gen_blogs %>% select(es.Genitiv, s.Genitiv, Lemma)
gen_blogs %>% select(Lemma:s.Genitiv)
#' You can also rename variables:
gen_blogs %>% select(Lemma, s_genitive = s.Genitiv,
                     es_genitive = es.Genitiv)
#' If you just want to rename a column while keeping all other
#' columns, `rename()` might be more practical:
gen_blogs %>% rename(s_genitive = s.Genitiv)
#' `select()` is also useful to change the order of columns:
gen_blogs %>% select(s.Genitiv, everything()) # everything(): helper function


#' #### Filtering data sets
#' You'll often want to get parts of a data set not according to
#' their position, but according to certain conditions which must
#' be fulfilled. That's what `filter()` is for.
#' 
#' gen_blogs has 17512 rows -- let's just use the lemmas which
#' appear at least five times in any form (arbitrary choice):
gen_blogs <- gen_blogs %>% filter(s.Genitiv >= 5 | es.Genitiv >= 5)
gen_blogs
 
#' If several conditions have to be fulfilled, they can be
#' separated by commas:
gen_blogs %>% filter(s.Genitiv >= 100, es.Genitiv >= 100)
#' Logical AND works the same way:
gen_blogs %>% filter(s.Genitiv >= 100 & es.Genitiv >= 100)

#' Try to ...
#' 
#' - select all rows where `s.Genitiv` is exactly 100
#' - select all rows where the lemma is not "Äußer"
#' - select all rows where es.Genitiv is between 100 und 200
#'   
#' Besides "Äußer", there are some other lemmas in the data set
#' which shouldn't be in there.  
#' Let's throw them out by using `%in%`:
gen_blogs <- gen_blogs %>% filter(!(Lemma %in%
                                      c("Äußer", "Inner", "Wichtiger", 
                                        "Schlimmer", "Besser", "Neu")))

#' More cleanup, using string functions and regular expressions:
#' 
#' Words ending in *-nis* have been improperly lemmatised (*-niss*):
str_subset(gen_blogs$Lemma, "niss$")
gen_blogs$Lemma <- str_replace(gen_blogs$Lemma, "niss$", "nis")
#' There are also very few lemmas with non-alphanumerical
#' characters at the end:
gen_blogs$Lemma <- str_replace(gen_blogs$Lemma, "[^[:alpha:]]$", "")

#' #### Adding columns
#' If you want to add a column to an existing data.frame, tibble
#' or data.table, the vector needs to have the same length as the
#' other columns.
#' 
#' There are quite a few ways to do this. Imho, the easiest one
#' is this:
gen_blogs$Length <- nchar(gen_blogs$Lemma) # Wortlänge in Buchstaben
gen_blogs

#' `mutate()` can be used to add several columns at once, to change
#' existing columns, and to do calculations with columns:
gen_blogs <- gen_blogs %>% mutate(Total = s.Genitiv + es.Genitiv,
                     Frac_es = round(es.Genitiv / Total, 2))
gen_blogs

# install.packages("sylly")
# install.packages("sylly.de", repo="https://undocumeantit.github.io/repos/l10n")
#' optional step: new column with the number of syllables
library(sylly.de)
gen_blogs$Syllables <- hyphen_c(gen_blogs$Lemma, hyph.pattern = "de")


#' #### Sorting
#' Use `arrange()` to change the order of rows:
gen_blogs %>% arrange(desc(es.Genitiv))
#' `desc()` to sort in descending order
#' 
#' You can also sort by several columns:
gen_blogs %>% arrange(Length, Lemma)
gen_blogs %>% arrange(desc(Length), desc(s.Genitiv), desc(es.Genitiv))

#' #### Summarising data
#' - `group_by()` creates a grouped tibble
#' - `summarise` is then used for arbitrary operations (sums,
#' means, standard deviations, ...) which are performed by group
gen_blogs %>% group_by(Length) %>% 
  summarise(Lemma_count = n(), s_genitives = sum(s.Genitiv), 
            es_genitives = sum(es.Genitiv))

gen_blogs %>% group_by(Syllables) %>% 
  summarise(Lemma_count = n(), s_genitives = sum(s.Genitiv), 
            es_genitives = sum(es.Genitiv))

#' Does the lemma end in s, ß, z or x?
gen_blogs$Ends_in_s <- factor(ifelse(str_sub(gen_blogs$Lemma, start = -1) %in% c("s", "ß", "z", "x"), "yes", "no"))
gen_blogs
gen_blogs %>% group_by(Ends_in_s) %>%
  summarise(s = sum(s.Genitiv), es = sum(es.Genitiv))