#' ---
#' title: "Plotting Data"
#' author: "Andreas Blombach, Philipp Heinrich, Stefan Evert"
#' date: "08.10.2019"
#' output:
#'  html_document:
#'    theme: readable
#'    highlight: tango
#'    toc: true
#'    toc_float: true
#'    fig_width: 10
#' ---
#' 

#' Loading required packages:
library(tidyverse)
library(wordcloud2)
library(psych)
library(GGally)
library(Hmisc)
library(ggcorrplot)
library(ggalt)

#' Good introduction to visualising data with ggplot/tidyverse
#' (by its creator, Hadley Wickham):
#' http://r4ds.had.co.nz/data-visualisation.html.
#' 

#' ## Setup
#' We'll work with different data sets here, but mainly with this
#' one:
collotask <- read_csv2("data/collocation_task.csv")

#' Study: Participants had to find matching collocates for a
#' number of German words (60 in total). For each word, they were
#' presented with four words from which they had to choose the
#' most likely collocate.
#' 
#' 
#' Subject: A number identifying each participant
#' 
#' Item: A number identifying each target word (1–60)
#' 
#' Accuracy: Was the participant's choice for this specific item
#' correct (1) or not (0)?
#' 
#' Word_class: The kind of collocation (ADJ + N, V + N, ADV + ADJ
#' or V + ADV)
#' 
#' SPR: Participants did a self-paced reading task where they had
#' to read a text on the laptop by pressing the space button. The
#' value of SPR is the total number of seconds that it took the
#' participant to complete the task.  
#' Expectation: SPR as a measure of reading habit which should
#' correlate with collocations scores in the paper-based test.
#' 
#' Age: What do you think?
#' 
#' Onset: Age when learners started learning German
#' 
#' Education: Academic qualification or not?
#' 
#' Majors: Languages or other majors?
#' 
#' Mother_tongue: Guess again.
#' 
#' Script: Is the writing system of the learner's mother tongue
#' the same as in German?
#' 
#' Read_DE: Weekly time spent reading in German for pleasure    
#' The questionnaire had 11 options:
#' never, < 15 min, 15-30min, 30min - 1 hour, 1-2 hours,
#' < 3 hours, etc.
#' 
#' These are coded as from 0 to 10.
#' 
#' Residence: Number of months spent in the L2 country (Germany)
collotask

#' Now that we've seen the full list of variables, let's drop
#' some. (For more advanced users who are already bored at this
#' point: Try to figure out *why* we drop them.)

collotask <- collotask %>%
  select(-Mother_tongue, -Education)
collotask

#' Turn the non-continuous variables into factors:
collotask$Subject <- factor(collotask$Subject)
collotask$Item <- factor(collotask$Item)
collotask$Word_class <- factor(collotask$Word_class,
                               levels = c(1, 2, 3, 4),
                               labels = c('ADJ + N', 'V + N', 'ADV + ADJ', 'V + ADV'))
collotask$Majors <- factor(collotask$Majors)
collotask$Script <- factor(collotask$Script)

collotask

#' N.B.: Accuracy should be a factor as well, but if we want to
#' look at a participant's overall accuracy, we need to be able
#' to calculate it. So we'll leave it for now.





#' Transform the data into "wide" format:
learners <- collotask %>%
  pivot_wider(names_from = Item, values_from = c(Accuracy, Word_class), names_prefix = "Item_")
learners

#' ## Exploring and visualising the data
#' 
#' ### Overview
ggpairs(collotask %>% select(-Subject, -Item))
ggpairs(learners %>% select(-Subject, -starts_with("Accuracy"),
                            -starts_with("Word_class")))

#' ### Bar plots
#' Bar plots show absolute or relative frequencies. Don't use
#' pie charts for this – while they may (arguably) be prettier,
#' bar plots are much easier to interpret, especially if a
#' variable has lots of levels.
ggplot(learners, aes(x = Majors)) +
  geom_bar()

ggplot(learners, aes(x = Read_DE)) +
  geom_bar()

#' You can also plot raw numbers, if you set `stat = "identity"`:
reading <- learners %>% group_by(Read_DE) %>%
  summarise(Freq = n()) %>%
  mutate(Percent = Freq / sum(Freq) * 100)

ggplot(reading, aes(x = Read_DE, y = Freq)) +
  geom_bar(stat = "identity")

ggplot(reading, aes(x = Read_DE, y = Percent)) +
  geom_bar(stat = "identity")

#' ### Histograms
#' For a histogram, the range of values is first divided into a
#' series of intervals (bins). Then, the values in each bin are
#' counted and plotted as bars.
ggplot(learners, aes(x = SPR)) +
  geom_histogram()

#' How the histogramm looks depends very much upon the number of
#' bins:
ggplot(learners, aes(x = SPR)) +
  geom_histogram(binwidth = 20)

breaks <- pretty(range(learners$SPR),
                 n = nclass.Sturges(learners$SPR),
                 min.n = 1)
ggplot(learners, aes(x = SPR)) +
  geom_histogram(breaks = breaks)

#' ### Density plots
#' A density plot is a smoothed, continuous form of a histogram.
#' The y-axis shows probability density (areas under the curve
#' are actual probabilities).
ggplot(learners, aes(x = SPR)) +
  geom_density()


#' ### Box plots
#' A box plot shows the median (thick line), the IQR (height of 
#' the box = 50 % of the data) and potential outliers (the height
#' of the whiskers is usually capped at 1.5 * IQR; points beyond
#' this range are classified as outliers).
#' 
#' Box plots are often used to compare groups.
ggplot(learners, aes(x = "", y = Age)) +
  geom_boxplot() +
  ggtitle("Age of participants") +
  xlab("")

ggplot(learners, aes(x = "", y = SPR)) +
  geom_boxplot() +
  labs(title = "Self-paced reading task",
       y = "Time in seconds")

#' Group comparison:
ggplot(learners, aes(x = Majors, y = SPR)) +
  geom_boxplot() +
  labs(title = "Self-paced reading task",
       y = "Time in seconds")

#' You can also use notched box plots (if notches don't overlap,
#' the difference between medians is statistically significant;
#' the reverse is not necessarily true, however):
ggplot(learners, aes(x = Majors, y = SPR)) +
  geom_boxplot(notch = TRUE) +
  labs(title = "Self-paced reading task",
       y = "Time in seconds")

collotask %>% group_by(Subject, Script) %>%
  summarise(Accuracy = mean(Accuracy)) %>%
  ggplot(aes(x = Script, y = Accuracy)) + geom_boxplot(varwidth = TRUE) +
  geom_point(alpha = .5)

#' You can include more variables by using `facet_wrap()`:
#' 
#' If we want to have a look at majors:
collotask %>% group_by(Subject, Word_class, Majors) %>%
  summarise(Accuracy = mean(Accuracy)) %>%
  ggplot(aes(x = Word_class, y = Accuracy)) +
  geom_boxplot() +
  facet_wrap(~ Majors)

#' Accuracy by word class, majors and script (varwidth = TRUE to reflect group size in the width of a box)
collotask %>% group_by(Subject, Word_class, Majors, Script) %>%
  summarise(Accuracy = mean(Accuracy)) %>%
  ggplot(aes(x = Word_class, y = Accuracy)) +
  geom_boxplot(varwidth = TRUE) +
  facet_wrap(~ Majors + Script)


#' ### Violin plots
#' Violin plots are the weird offspring of box plots and
#' histograms.
ggplot(learners, aes(x = Majors, y = SPR)) +
  geom_violin() +
  labs(title = "Self-paced reading task",
       y = "Time in seconds")

#' ### Dot charts
#' Dot charts are a good way to visualise means, medians or other
#' descriptive statistics:
collotask %>% ggplot(aes(x = Word_class, y = Accuracy)) +
  stat_summary(fun.y = mean, geom = "point", na.rm = TRUE) +
  stat_summary(fun.data = mean_cl_normal, geom = "errorbar", width = .3) + # mean_se for standard errors instead
  geom_vline(xintercept = 1, linetype = 2, colour = rgb(0, 0, 0, .5)) +
  geom_vline(xintercept = 2, linetype = 2, colour = rgb(0, 0, 0, .5)) +
  geom_vline(xintercept = 3, linetype = 2, colour = rgb(0, 0, 0, .5)) +
  geom_vline(xintercept = 4, linetype = 2, colour = rgb(0, 0, 0, .5)) +
  theme_bw() +
  labs(title = "Mean accuracy (95% CI), learners of German",
       x = "Word class")

collotask %>% group_by(Subject, Word_class) %>% summarise(Accuracy = mean(Accuracy)) %>%
  ggplot(aes(x = Word_class, y = Accuracy)) +
  stat_summary(fun.y = mean, geom = "point", na.rm = TRUE) +
  stat_summary(fun.data = mean_cl_normal, geom = "errorbar", width = .3) +
  geom_vline(xintercept = 1, linetype = 2, colour = rgb(0, 0, 0, .5)) +
  geom_vline(xintercept = 2, linetype = 2, colour = rgb(0, 0, 0, .5)) +
  geom_vline(xintercept = 3, linetype = 2, colour = rgb(0, 0, 0, .5)) +
  geom_vline(xintercept = 4, linetype = 2, colour = rgb(0, 0, 0, .5)) +
  theme_bw() +
  labs(title = "Mean accuracy (95% CI), learners of German",
       x = "Word class")

#' Box plots might be more informative in this case, though. If
#' you really wanted to, you could still add confidence intervals ...
collotask %>% group_by(Subject, Word_class) %>% summarise(Accuracy = mean(Accuracy)) %>%
  ggplot(aes(x = Word_class, y = Accuracy)) +
  geom_boxplot(notch = TRUE) +
  labs(title = "Accuracy by word class, learners of German",
       x = "Word class")


#' ### Scatter plots
lexdiv <- read_tsv("data/diversity.tsv")

lowbrow <- lexdiv %>% filter(category == "lowbrow")

ggplot(data = lexdiv,
       aes(x = sichel_s, y = mtld, shape = category)) +
  geom_point(aes(col = genre), alpha = .7, size = 2) +
  geom_encircle(aes(x = sichel_s, y = mtld), data = lowbrow,
                expand = .01) +
  scale_color_brewer(palette = "Set2") +
  labs(title = "Combining two lexical diversity measures",
       x = "Sichel's S", y = "MTLD")

#' ### Correlation matrices
#' Let's think about "Onset", "Age" and "Residence":
#' 
#' - How useful are "Onset" and "Age" as separate predictors?
#' - Would something like "years spent learning the language"
#' be more useful?
#' - How high are the correlations between these variables?
collotask <- collotask %>% mutate(Years_learning = Age - Onset)

collotask %>% select(Onset, Age, Residence, Years_learning) %>%
  cor() %>% ggcorrplot(lab = TRUE) +
  ggtitle("Pearson correlations")


#' ### Themes
#' There are different themes to change the appearance of ggplots.
#' 
#' Just add the corresponding function as another layer to your
#' call of `ggplot` (e.g. `+ theme_dark()` or `+ theme_minimal()`).
#' 
#' An overview of themes can be found here:
#' https://ggplot2.tidyverse.org/reference/ggtheme.html
#' 
#' The package `ggthemes` adds even more themes.